Attribute VB_Name = "UPCEAN"
' VB / VBA Functions for UPC/EAN
' Copyright 2004 by MW6 Technologies Inc. All rights reserved.
'
' This code may not be modified or distributed unless you purchase
' the license from MW6.

Private I As Integer
Private StrLen As Integer
Private CheckSum As Integer
Private Data As String
Private CharSet As String
Private CurrChar As Integer
Private Message As String
Private Weight  As Integer
Private CheckDigit As Integer

Public Function GetCheckDigit(ByVal Src As String) As Integer
    StrLen = Len(Src)
    CheckSum = 0
    For I = 1 To StrLen
        If I Mod 2 = 1 Then
            CheckSum = CheckSum + (Asc(Mid(Src, StrLen - I + 1, 1)) - Asc("0")) * 3
        Else
            CheckSum = CheckSum + Asc(Mid(Src, StrLen - I + 1, 1)) - Asc("0")
        End If
    Next I
    
    CheckSum = CheckSum Mod 10
    If CheckSum <> 0 Then
        CheckSum = 10 - CheckSum
    End If
    GetCheckDigit = CheckSum
    
End Function

Public Function EAN13(ByVal Src As String) As String
    Dim tmp As String
    Data = Src
    StrLen = Len(Src)
    
    ' check length to make sure Data has 12 characters
    If StrLen < 12 Then
        While Len(Data) < 12
            Data = Data & "0"
        Wend
    ElseIf StrLen > 12 Then
        Data = Left(Data, 12)
    End If

    ' choose character set depending on the first character of string
    Select Case (Asc(Mid(Data, 1, 1)) - Asc("0"))
        Case 0: CharSet = "AAAAAA"
        Case 1: CharSet = "AABABB"
        Case 2: CharSet = "AABBAB"
        Case 3: CharSet = "AABBBA"
        Case 4: CharSet = "ABAABB"
        Case 5: CharSet = "ABBAAB"
        Case 6: CharSet = "ABBBAA"
        Case 7: CharSet = "ABABAB"
        Case 8: CharSet = "ABABBA"
        Case 9: CharSet = "ABBABA"
    End Select

    Message = Chr(Asc(Mid(Data, 1, 1)) - Asc("0") + 192) & "("
    For I = 2 To 7
        tmp = Mid(CharSet, I - 1, 1)
        If tmp = "A" Then
            Message = Message & Mid(Data, I, 1)
        ElseIf tmp = "B" Then
            Message = Message & Chr(Asc(Mid(Data, I, 1)) - Asc("0") + Asc("Q"))
        End If
    Next I
    
    Message = Message & "*"
    For I = 8 To 12
        Message = Message & Chr(Asc(Mid(Data, I, 1)) - Asc("0") + Asc("A"))
    Next I
    
    Message = Message & Chr(GetCheckDigit(Data) + Asc("A"))
    Message = Message & ")"
    
    EAN13 = Message
End Function

Public Function EAN8(ByVal Src As String) As String
    StrLen = Len(Src)
    Data = Src
    
    ' check length to make sure Data has 7 characters
    If StrLen < 7 Then
        While Len(Data) < 7
            Data = Data & "0"
        Wend
    ElseIf StrLen > 7 Then
        Data = Left(Data, 7)
    End If

    Message = "("
    
    For I = 1 To 4
        Message = Message & Mid(Data, I, 1)
    Next I
    
    Message = Message & "*"
    For I = 5 To 7
        Message = Message & Chr(Asc(Mid(Data, I, 1)) - Asc("0") + Asc("A"))
    Next I
    
    Message = Message & Chr(GetCheckDigit(Data) + Asc("A"))
    Message = Message & ")"
    
    EAN8 = Message
End Function

Public Function UPCA(ByVal Src As String) As String
    StrLen = Len(Src)
    Data = Src
    
    ' check length to make sure Data has 11 characters
    If StrLen < 11 Then
        While Len(Data) < 11
            Data = Data & "0"
        Wend
    ElseIf StrLen > 11 Then
        Data = Left(Data, 11)
    End If
    
    Message = Chr(Asc(Mid(Data, 1, 1)) - Asc("0") + 192) & "("
    Message = Message & Chr(Asc(Mid(Data, 1, 1)) - Asc("0") + Asc("a"))
     
    For I = 2 To 6
        Message = Message & Mid(Data, I, 1)
    Next I
    
    Message = Message & "*"
    For I = 7 To 11
        Message = Message & Chr(Asc(Mid(Data, I, 1)) - Asc("0") + Asc("A"))
    Next I
    
    CheckDigit = GetCheckDigit(Data)
    Message = Message & Chr(CheckDigit + Asc("q"))
    Message = Message & ")" & Chr(CheckDigit + 192)
    
    UPCA = Message
End Function

Public Function UPCE(ByVal Src As String) As String
    Dim tmp As String
    
    StrLen = Len(Src)
    Data = Src
    
    ' check length to make sure Data has 6 characters
    If StrLen < 6 Then
        While Len(Data) < 6
            Data = Data & "0"
        Wend
    ElseIf StrLen > 6 Then
        Data = Left(Src, 6)
    End If
    Data = "0" & Data

    ' choose character set depending on the check digit value
    CheckDigit = GetCheckDigit(UPCE2UPCA(Data))
    Select Case CheckDigit
        Case 0: CharSet = "BBBAAA"
        Case 1: CharSet = "BBABAA"
        Case 2: CharSet = "BBAABA"
        Case 3: CharSet = "BBAAAB"
        Case 4: CharSet = "BABBAA"
        Case 5: CharSet = "BAABBA"
        Case 6: CharSet = "BAAABB"
        Case 7: CharSet = "BABABA"
        Case 8: CharSet = "BABAAB"
        Case 9: CharSet = "BAABAB"
    End Select

    Message = Chr(192) & "("
    For I = 2 To 7
        tmp = Mid(CharSet, I - 1, 1)
        If tmp = "A" Then
            Message = Message & Mid(Data, I, 1)
        ElseIf tmp = "B" Then
            Message = Message & Chr(Asc(Mid(Data, I, 1)) - Asc("0") + Asc("Q"))
        End If
    Next I
    Message = Message & "+" & Chr(CheckDigit + 192)
    
    UPCE = Message
End Function

Public Function UPCE2UPCA(ByVal Src As String) As String
    Select Case Mid(Src, 7, 1)
        Case "0":
            UPCE2UPCA = Mid(Src, 1, 3) + Mid(Src, 7, 1) + "0000" + Mid(Src, 4, 3)
        Case "1":
            UPCE2UPCA = Mid(Src, 1, 3) + Mid(Src, 7, 1) + "0000" + Mid(Src, 4, 3)
        Case "2":
            UPCE2UPCA = Mid(Src, 1, 3) + Mid(Src, 7, 1) + "0000" + Mid(Src, 4, 3)
        Case "3":
            UPCE2UPCA = Mid(Src, 1, 4) + "00000" + Mid(Src, 5, 2)
        Case "4":
            UPCE2UPCA = Mid(Src, 1, 5) + "00000" + Mid(Src, 6, 1)
        Case "5":
            UPCE2UPCA = Mid(Src, 1, 6) + "0000" + Mid(Src, 7, 1)
        Case "6":
            UPCE2UPCA = Mid(Src, 1, 6) + "0000" + Mid(Src, 7, 1)
        Case "7":
            UPCE2UPCA = Mid(Src, 1, 6) + "0000" + Mid(Src, 7, 1)
        Case "8":
            UPCE2UPCA = Mid(Src, 1, 6) + "0000" + Mid(Src, 7, 1)
        Case "9":
            UPCE2UPCA = Mid(Src, 1, 6) + "0000" + Mid(Src, 7, 1)
    End Select
End Function

