using System;

namespace Demo
{
	/// <summary>
	/// Summary description for Code128.
	/// </summary>
	public class UPC_EAN
	{
		private string CharSet;
		private string Message;

		public UPC_EAN()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		private string UPCE2UPCA(string Src)
		{
			char[] buf = Src.ToCharArray();
			string UPCE2UPCA = "";
			switch (buf[6])
			{
				case '0':
				case '1':
				case '2':
					UPCE2UPCA = Src.Substring(0, 3) + Src.Substring(6, 1) + "0000" + Src.Substring(3, 3);
					break;

				case '3':
					UPCE2UPCA = Src.Substring(0, 4) + "00000" + Src.Substring(4, 2);
					break;

				case '4':
					UPCE2UPCA = Src.Substring(0, 5) + "00000" + Src.Substring(5, 1);
					break;

				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
					UPCE2UPCA = Src.Substring(0, 6) + "0000" + Src.Substring(6, 1);
					break;
			}

			return UPCE2UPCA;
		}


		private int GetCheckDigit(string Src)
		{
			char[] buf = Src.ToCharArray();
			int StrLen = Src.Length;
			int CheckSum = 0;
			for (int i = 1; i <= StrLen; i++)
			{
				if (i % 2 == 1)
					CheckSum += (buf[StrLen - i] - '0') * 3;
				else
					CheckSum += + buf[StrLen - i] - '0';
			}

			CheckSum = CheckSum % 10;
			if (CheckSum != 0) 
				CheckSum = 10 - CheckSum;
			
			return CheckSum;
		}

		public string EAN13(string Src)
		{
			string Data = Src;
			int StrLen = Src.Length; 

			// check length to make sure Data has 12 characters
			if (StrLen < 12)
			{
				while (Data.Length < 12)
				{
					Data = Data + "0";
				}
			}
			else if (StrLen > 12)
			{
				Data = Data.Substring(0, 12);
			}

			char[] buf = Data.ToCharArray(); 
			// choose character set depending on the first character of string
			switch (buf[0] - '0')
			{
				case 0: CharSet = "AAAAAA"; break;
				case 1: CharSet = "AABABB"; break;
				case 2: CharSet = "AABBAB"; break;
				case 3: CharSet = "AABBBA"; break;
				case 4: CharSet = "ABAABB"; break;
				case 5: CharSet = "ABBAAB"; break;
				case 6: CharSet = "ABBBAA"; break;
				case 7: CharSet = "ABABAB"; break;
				case 8: CharSet = "ABABBA"; break;
				case 9: CharSet = "ABBABA"; break;
			}
			char[] buf_1 = CharSet.ToCharArray(); 

			Message = "" + (char)(buf[0] - '0' + 192) + "(";
			
			for(int i = 2; i <= 7; i++)
			{
				if (buf_1[i - 2] == 'A')
					Message = Message + buf[i - 1];
				else if (buf_1[i - 2] == 'B')
					Message = Message + (char)(buf[i - 1] - '0' + 'Q');
			}

			Message = Message + "*";
			for(int i = 8; i <= 12; i++)
			{
				Message = Message + (char)(buf[i - 1] - '0' + 'A');
			}

			Message = Message + (char)(GetCheckDigit(Data) + 'A');
			Message = Message + ")";

			return Message;
		}

		public string EAN8(string Src)
		{
			string Data = Src;
			int StrLen = Src.Length; 

			// check length to make sure Data has 7 characters
			if (StrLen < 7)
			{
				while (Data.Length < 7)
				{
					Data = Data + "0";
				}
			}
			else if (StrLen > 7)
			{
				Data = Data.Substring(0, 12);
			}

			char[] buf = Data.ToCharArray(); 

			Message = "(";
			
			for(int i = 1; i <= 4; i++)
			{
				Message = Message + buf[i - 1];
			}

			Message = Message + "*";
			for(int i = 5; i <= 7; i++)
			{
				Message = Message + (char)(buf[i - 1] - '0' + 'A');
			}

			Message = Message + (char)(GetCheckDigit(Data) + 'A');
			Message = Message + ")";

			return Message;
		}

		public string UPCA(string Src)
		{
			string Data = Src;
			int StrLen = Src.Length; 

			// check length to make sure Data has 11 characters
			if (StrLen < 11)
			{
				while (Data.Length < 11)
				{
					Data = Data + "0";
				}
			}
			else if (StrLen > 11)
			{
				Data = Data.Substring(0, 11);
			}

			char[] buf = Data.ToCharArray(); 

			Message = "" + (char)(buf[0] - '0' + 192);
			Message = Message + "(";
			Message = Message + (char)(buf[0] - '0' + 'a');
			
			for(int i = 2; i <= 6; i++)
			{
				Message = Message + buf[i - 1];
			}

			Message = Message + "*";
			for(int i = 7; i <= 11; i++)
			{
				Message = Message + (char)(buf[i - 1] - '0' + 'A');
			}

			int CheckDigit = GetCheckDigit(Data);
			Message = Message + (char)(CheckDigit + 'q');
			Message = Message + ")" + (char)(CheckDigit + 192);

			return Message;
		}

		public string UPCE(string Src)
		{
			string Data = Src;
			int StrLen = Src.Length; 

			// check length to make sure Data has 6 characters
			if (StrLen < 6)
			{
				while (Data.Length < 6)
				{
					Data = Data + "0";
				}
			}
			else if (StrLen > 6)
			{
				Data = Data.Substring(0, 6);
			}
			Data = "0" + Data;

			char[] buf = Data.ToCharArray(); 
			int CheckDigit = GetCheckDigit(UPCE2UPCA(Data));
			// choose character set depending on the first character of string
			switch (CheckDigit)
			{
				case 0: CharSet = "BBBAAA"; break;
				case 1: CharSet = "BBABAA"; break;
				case 2: CharSet = "BBAABA"; break;
				case 3: CharSet = "BBAAAB"; break;
				case 4: CharSet = "BABBAA"; break;
				case 5: CharSet = "BAABBA"; break;
				case 6: CharSet = "BAAABB"; break;
				case 7: CharSet = "BABABA"; break;
				case 8: CharSet = "BABAAB"; break;
				case 9: CharSet = "BAABAB"; break;
			}
			char[] buf_1 = CharSet.ToCharArray(); 

			Message = "" + (char)192 + "(";
			
			for(int i = 2; i <= 7; i++)
			{
				if (buf_1[i - 2] == 'A')
					Message = Message + buf[i - 1];
				else if (buf_1[i - 2] == 'B')
					Message = Message + (char)(buf[i - 1] - '0' + 'Q');
			}

			Message = Message + "+" + (char)(CheckDigit + 192);

			return Message;
		}
	}
}
